//
//  PSMSlideshow.h
//  PSMSlideshow
//
//  Created by John Pannell on 9/14/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

/*
This is a controller object that manages the details of presenting various images to the user in a slideshow format. It also provides the view for display of the slideshow.
 
It leverages Quartz Composer and the QCView object for design and display of slideshows. 
*/

#import <Cocoa/Cocoa.h>
#import <Quartz/Quartz.h>
#import "PSMSlideshowDataSource.h"

@class PSMSlideshowImage;


@interface PSMSlideshow : NSObject {

    // config options
    float                       transitionDuration;
    float                       displayDuration;
    BOOL                        useKenBurnsEffect;
    BOOL                        scaleToFill;
    BOOL                        randomizeImages;
    BOOL                        autoAdvance;
    BOOL                        repeatShow;
    NSString                    *transitionName;
    int                         transitionDirection;
    
    // internal state
    id<PSMSlideshowDataSource>  slides;                 
    NSMutableArray              *viewedSlideIndices;    // for randomization
    int                         slideIndex;             // points to slide that is "on deck", internal
    
    // transition timing
    NSTimer                     *autoAdvanceTimer;
    NSTimer                     *animationTimer;
    BOOL                        nextSlideIsReady;
    BOOL                        isPaused;
    float                       currentTransitionTime;
    NSDate                      *referenceDate;
    
    PSMSlideshowImage           *incomingImage;
    PSMSlideshowImage           *outgoingImage;
    
    QCView                      *slideshowView;
    
}

// class methods
+ (NSArray *)availableTransitionNames;
+ (NSArray *)directionOptionsForTransitionName:(NSString *)transitionName;

// creation/destruction
- (id)initWithImagePaths:(NSArray *)paths;
- (id)initWithContentsOfFolder:(NSString *)path includeSubfolders:(BOOL)subFlag;
- (id)initWithDataSource:(id <PSMSlideshowDataSource>)dataSource;

// accessors
- (float)transitionDuration;
- (void)setTransitionDuration:(float)value;
- (float)displayDuration;
- (void)setDisplayDuration:(float)value;
- (BOOL)useKenBurnsEffect;
- (void)setUseKenBurnsEffect:(BOOL)flag;
- (BOOL)scaleToFill;
- (void)setScaleToFill:(BOOL)flag;
- (BOOL)randomizeImages;
- (void)setRandomizeImages:(BOOL)flag;
- (BOOL)autoAdvance;
- (void)setAutoAdvance:(BOOL)flag;
- (BOOL)repeatShow;
- (void)setRepeatShow:(BOOL)flag;
- (NSString *)transitionName;
- (void)setTransitionName:(NSString *)name;
- (int)transitionDirection;
- (void)setTransitionDirection:(int)value;
- (int)slideCount;
- (int)currentSlideIndex;
- (BOOL)isPaused;

// control
- (void)load;
- (void)rewind;
- (void)pause;
- (void)resume;
- (void)showNextSlide;
- (void)showPreviousSlide;

// something to show
- (QCView *)slideshowView;

@end
