//
//  PSMSlideshowInternalDataSource.m
//  PSMSlideshow
//
//  Created by John Pannell on 9/29/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import <Quartz/Quartz.h>
#import "PSMSlideshowInternalDataSource.h"

@interface PSMSlideshowInternalDataSource (Private)
- (BOOL)isReadableImageAtPath:(NSString *)path;
@end

@implementation PSMSlideshowInternalDataSource

#pragma mark -
#pragma mark Creation/Destruction

- (id)initWithFilePaths:(NSArray *)paths
{
    if(self = [super init]){
        images = [[NSMutableArray alloc] init];

        // preflight files - are they legit paths that represent readable images?
        NSFileManager *fm = [NSFileManager defaultManager];
        BOOL isDir = NO;
        NSEnumerator *e = [paths objectEnumerator];
        NSString *path;
        while(path = [e nextObject]){
            if([fm fileExistsAtPath:path isDirectory:&isDir] && !isDir){
                if ([self isReadableImageAtPath:path]){
                    [images addObject:path];
                }
            }
        }
    }
    return self;
}

- (id)initWithTestPaths:(NSArray *)paths storageFormat:(int)value
{
    if(self = [super init]){
        images = [[NSMutableArray alloc] init];
        
        // preflight files - are they legit paths that represent readable images?
        NSFileManager *fm = [NSFileManager defaultManager];
        BOOL isDir = NO;
        NSEnumerator *e = [paths objectEnumerator];
        NSString *path;
        while(path = [e nextObject]){
            if([fm fileExistsAtPath:path isDirectory:&isDir] && !isDir){
                switch(value){ // don't do this with your datasource - just pick a format
                    
                    case kPSMStoreAsPath:
                        if ([self isReadableImageAtPath:path]){
                            [images addObject:path];
                        }
                        break;
                        
                    case kPSMStoreAsNSURL:
                        if ([self isReadableImageAtPath:path]){
                            [images addObject:[NSURL fileURLWithPath:path]];
                        }
                        break;
                        
                    case kPSMStoreAsCGImage:
                        if ([self isReadableImageAtPath:path]){
                            CGImageRef image = NULL;
                            CGImageSourceRef imageSource = NULL;
                            
                            imageSource = CGImageSourceCreateWithURL((CFURLRef)[NSURL fileURLWithPath:path], NULL);
                            if(!imageSource){
                                NSLog(@"Could not create image source from URL");
                                return NULL;
                            }
                            
                            image = CGImageSourceCreateImageAtIndex(imageSource,0,NULL);
                            CFRelease(imageSource);
                            [images addObject:image];
                            // this might leak CGImage objects in actual use
                        }
                        break;
                        
                    case kPSMStoreAsNSImage:
                        if([[NSImage imageUnfilteredFileTypes] containsObject:[path pathExtension]]){
                            [images addObject:[[[NSImage alloc] initWithContentsOfFile:path] autorelease]];
                        }
                        break;
                        
                    case kPSMStoreAsNSFileWrapper:
                        if ([self isReadableImageAtPath:path]){
                            [images addObject:[[[NSFileWrapper alloc] initWithPath:path] autorelease]];
                        }
                        break;
                        
                    case kPSMStoreAsPDFPage:
                        if([[path pathExtension] isEqualToString:@"pdf"]){ // probably a better way to check this
                            PDFDocument *doc = [[PDFDocument alloc] initWithURL:[NSURL fileURLWithPath:path]];
                            int i, pageCount = [doc pageCount];
                            for(i = 0; i < pageCount; i++){
                                [images addObject:[doc pageAtIndex:i]];
                            }
                        }
                        break;
                        
                }
            }
        }
    }
    return self;
}


#pragma mark -
#pragma mark Formal Protocol

- (id)slideshowItemAtIndex:(int)index
{
    return [images objectAtIndex:index];
}

- (int)numberOfSlideshowItems
{
    return [images count];
}

#pragma mark -
#pragma mark Informal Protocol

- (NSString*)nameOfSlideshowItemAtIndex:(int)index
{
    return [[images objectAtIndex:index] lastPathComponent];
}

#pragma mark -
#pragma mark Utility

- (BOOL)isReadableImageAtPath:(NSString *)path
{
    NSArray *readableUTIs = (NSArray *)CGImageSourceCopyTypeIdentifiers();
    
    // get file at path
    MDItemRef item = MDItemCreate(kCFAllocatorDefault, (CFStringRef)path);
    if(!item){
        [readableUTIs release];
        return NO;
    }
    
    // determine UTI of this item
    CFTypeRef itemType = MDItemCopyAttribute(item, CFSTR("kMDItemContentType"));
    if(!itemType){
        CFRelease(item);
        [readableUTIs release];
        return NO;
    }
    
    // compare found UTI with readable UTIs in ImageIO
    NSEnumerator *e = [readableUTIs objectEnumerator];
    CFStringRef readableType;
    while(readableType = (CFStringRef)[e nextObject]){
        if(UTTypeEqual(readableType,itemType)){
            CFRelease(itemType);
            CFRelease(item);
            [readableUTIs release];
            return YES;
        }
    }
    
    // no matching result
    CFRelease(itemType);
    CFRelease(item);
    [readableUTIs release];
    return NO;
}
@end
