//
//  WindowController.m
//  PSMSlideshow
//
//  Created by John Pannell on 9/15/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "WindowController.h"
#import "PSMSlideshow.h"


@implementation WindowController

- (void)awakeFromNib
{
    // run an open panel to select directory of images
    NSOpenPanel *op = [NSOpenPanel openPanel];
    [op setCanChooseFiles:NO];
    [op setCanChooseDirectories:YES];
    [op setTitle:@"Select a directory of image files"];
    [op setAccessoryView:accessoryView];
    [op runModal];
    
    // open config drawer
    [configDrawer setMinContentSize:[configView frame].size];
    [configDrawer setMaxContentSize:[configView frame].size];
    [configDrawer toggle:self];
    
    slideshow = [[PSMSlideshow alloc] initWithContentsOfFolder:[op filename] includeSubfolders:[includeSubfolders state]];
    
    // install toolbar
    toolbar = [[NSToolbar alloc] initWithIdentifier:@"DemoToolbarC"];
    [toolbar setDelegate:self];
    [toolbar setAllowsUserCustomization:NO];
    [toolbar setAutosavesConfiguration:YES];
    [[self window] setToolbar:toolbar];
    
    // populate various prefs - transition names
    [self setTransitionNames:[PSMSlideshow availableTransitionNames]];
    [transitionPref bind:@"content" toObject:self withKeyPath:@"transitionNames" options:nil];
    [transitionPref bind:@"selectedObject" toObject:slideshow withKeyPath:@"transitionName" options:nil];
    
    // transition directions
    [self setDirectionOptions:[PSMSlideshow directionOptionsForTransitionName:[slideshow transitionName]]];
    [directionPref bind:@"content" toObject:self withKeyPath:@"directionOptions" options:nil];
    [directionPref bind:@"enabled" toObject:self withKeyPath:@"directionOptions" options:[NSDictionary dictionaryWithObject:NSIsNotNilTransformerName forKey:@"NSValueTransformerName"]];
    [directionPref bind:@"contentValues" toObject:self withKeyPath:@"directionOptions.title" options:nil];
    [directionPref bind:@"contentObjects" toObject:self withKeyPath:@"directionOptions.value" options:nil];
    [directionPref bind:@"selectedObject" toObject:slideshow withKeyPath:@"transitionDirection" options:nil];
    
    // bind prefs to slideshow object
    [scaleToFillPref bind:@"value" toObject:slideshow withKeyPath:@"scaleToFill" options:nil];
    [transitionDurationPref bind:@"value" toObject:slideshow withKeyPath:@"transitionDuration" options:nil];
    [autoAdvancePref bind:@"value" toObject:slideshow withKeyPath:@"autoAdvance" options:nil];
    [displayDurationPref bind:@"value" toObject:slideshow withKeyPath:@"displayDuration" options:nil];
    [randomizeImagesPref bind:@"value" toObject:slideshow withKeyPath:@"randomizeImages" options:nil];
    [repeatShowPref bind:@"value" toObject:slideshow withKeyPath:@"repeatShow" options:nil];
    [useKenBurnsEffectPref bind:@"enabled" toObject:slideshow withKeyPath:@"autoAdvance" options:nil];
    [useKenBurnsEffectPref bind:@"value" toObject:slideshow withKeyPath:@"useKenBurnsEffect" options:nil];
    
    // create slideshow and assign view
    [[self window] setContentView:[slideshow slideshowView]];
    [slideshow load];
}

#pragma mark -
#pragma mark Slideshow Actions

- (IBAction)rewind:(id)sender
{
    [slideshow rewind];
}

- (IBAction)pause:(id)sender
{
    [slideshow pause];
}

- (IBAction)resume:(id)sender
{
    [slideshow resume];
}

- (IBAction)showNextSlide:(id)sender
{
    [slideshow showNextSlide];
}

- (IBAction)showPreviousSlide:(id)sender
{
    [slideshow showPreviousSlide];
}

#pragma mark -
#pragma mark Toolbar

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag {
    NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
    
    if([itemIdentifier isEqualToString:@"PreviousItem"]){
        [item setPaletteLabel:@"Previous"]; 
        [item setLabel:@"Previous"]; 
        [item setToolTip:@"View previous slide"];
        [item setImage:[NSImage imageNamed:@"previous"]];
        [item setTarget:self]; 
        [item setAction:@selector(showPreviousSlide:)];
        
    } else if([itemIdentifier isEqualToString:@"PlayItem"]){
        [item setPaletteLabel:@"Play"]; 
        [item setLabel:@"Play"]; 
        [item setToolTip:@"Play slideshow"];
        [item setImage:[NSImage imageNamed:@"Play"]];
        [item setTarget:self]; 
        [item setAction:@selector(resume:)];
        
    } else if([itemIdentifier isEqualToString:@"PauseItem"]){
        [item setPaletteLabel:@"Pause"]; 
        [item setLabel:@"Pause"]; 
        [item setToolTip:@"Pause slideshow"];
        [item setImage:[NSImage imageNamed:@"Pause"]];
        [item setTarget:self]; 
        [item setAction:@selector(pause:)];
        
    } else if([itemIdentifier isEqualToString:@"NextItem"]){
        [item setPaletteLabel:@"Next"]; 
        [item setLabel:@"Next"]; 
        [item setToolTip:@"View next slide"];
        [item setImage:[NSImage imageNamed:@"next"]];
        [item setTarget:self]; 
        [item setAction:@selector(showNextSlide:)];
        
    } else if([itemIdentifier isEqualToString:@"RewindItem"]){
        [item setPaletteLabel:@"Rewind"]; 
        [item setLabel:@"Rewind"]; 
        [item setToolTip:@"Start show from beginning"];
        [item setImage:[NSImage imageNamed:@"Start"]];
        [item setTarget:self]; 
        [item setAction:@selector(rewind:)];
        
    }
    
    return [item autorelease];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar {
    return [NSArray arrayWithObjects:
        @"RewindItem",
        @"PreviousItem",
        NSToolbarFlexibleSpaceItemIdentifier,
        @"PlayItem",
        @"PauseItem",
        NSToolbarFlexibleSpaceItemIdentifier,
        @"NextItem",
        nil];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar {
    return [NSArray arrayWithObjects: 
        @"RewindItem",
        @"PreviousItem",
        NSToolbarFlexibleSpaceItemIdentifier,
        @"PlayItem",
        @"PauseItem",
        @"NextItem",
        nil];
}

- (IBAction)customizeToolbar:(id)sender {
    [toolbar runCustomizationPalette:sender];
}

- (IBAction)toggleToolbar:(id)sender {
    [toolbar setVisible:![toolbar isVisible]];
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
    NSString *itemIdentifier = [theItem itemIdentifier];
    
    if([itemIdentifier isEqualToString:@"PreviousItem"]){
        if([slideshow currentSlideIndex] > 1)
            return YES;
        
    } else if([itemIdentifier isEqualToString:@"PlayItem"]){
        if([slideshow isPaused])
            return YES;
        
    } else if([itemIdentifier isEqualToString:@"PauseItem"]){
        if(![slideshow isPaused])
            return YES;
        
    } else if([itemIdentifier isEqualToString:@"NextItem"]){
        if(([slideshow currentSlideIndex] <= [slideshow slideCount] - 1) || [slideshow repeatShow])
            return YES;
        
    } else if([itemIdentifier isEqualToString:@"RewindItem"]){
        if([slideshow currentSlideIndex] > 1)
            return YES;
    }
    
    return NO;
}

#pragma mark -
#pragma mark Prefs

- (NSArray *)directionOptions
{
    return directionOptions;
}

- (void)setDirectionOptions:(NSArray *)options
{
    [options retain];
    [directionOptions release];
    directionOptions = options;
}

- (NSArray *)transitionNames
{
    return transitionNames;
}

- (void)setTransitionNames:(NSArray *)names
{
    [names retain];
    [transitionNames release];
    transitionNames = names;
}

- (IBAction)changeTransition:(id)sender
{
    // must reload direction options
    [self setDirectionOptions:[PSMSlideshow directionOptionsForTransitionName:[sender stringValue]]];
}

@end
